@@ -32,23 +32,36 @@ SPDX3_CONTEXT_URL = "https://spdx.github.io/spdx-3-model/context.json"
 TEST_TZ = timezone(timedelta(hours=-2), name="TST")
 
 
-@pytest.fixture(scope="module")
-def test_context(tmp_path_factory, model_server):
-    tmp_directory = tmp_path_factory.mktemp("pythontestcontext")
-    outfile = tmp_directory / "model.py"
-    subprocess.run(
+def shacl2code_generate(args, outfile):
+    return subprocess.run(
         [
             "shacl2code",
             "generate",
-            "--input",
-            TEST_MODEL,
-            "--context",
-            model_server + "/test-context.json",
+        ]
+        + args
+        + [
             "python",
             "--output",
             outfile,
         ],
         check=True,
+        stdout=subprocess.PIPE,
+        encoding="utf-8",
+    )
+
+
+@pytest.fixture(scope="module")
+def test_context(tmp_path_factory, model_server):
+    tmp_directory = tmp_path_factory.mktemp("pythontestcontext")
+    outfile = tmp_directory / "model.py"
+    shacl2code_generate(
+        [
+            "--input",
+            TEST_MODEL,
+            "--context",
+            model_server + "/test-context.json",
+        ],
+        outfile,
     )
     outfile.chmod(0o755)
     yield (tmp_directory, outfile)
@@ -83,24 +96,16 @@ def model(test_context):
     ],
 )
 class TestOutput:
+    """
+    Test syntax and formatting of the output file
+    """
+
     def test_output_syntax(self, tmp_path, args):
         """
         Checks that the output file is valid python syntax by executing it"
         """
         outfile = tmp_path / "output.py"
-        subprocess.run(
-            [
-                "shacl2code",
-                "generate",
-            ]
-            + args
-            + [
-                "python",
-                "--output",
-                outfile,
-            ],
-            check=True,
-        )
+        shacl2code_generate(args, outfile)
 
         subprocess.run([sys.executable, outfile, "--help"], check=True)
 
@@ -108,21 +113,7 @@ class TestOutput:
         """
         Tests that the generated file does not have trailing whitespace
         """
-        p = subprocess.run(
-            [
-                "shacl2code",
-                "generate",
-            ]
-            + args
-            + [
-                "python",
-                "--output",
-                "-",
-            ],
-            check=True,
-            stdout=subprocess.PIPE,
-            encoding="utf-8",
-        )
+        p = shacl2code_generate(args, "-")
 
         for num, line in enumerate(p.stdout.splitlines()):
             assert (
@@ -133,43 +124,79 @@ class TestOutput:
         """
         Tests that the output file doesn't contain tabs
         """
-        p = subprocess.run(
-            [
-                "shacl2code",
-                "generate",
-            ]
-            + args
-            + [
-                "python",
-                "--output",
-                "-",
-            ],
-            check=True,
-            stdout=subprocess.PIPE,
-            encoding="utf-8",
-        )
+        p = shacl2code_generate(args, "-")
 
         for num, line in enumerate(p.stdout.splitlines()):
             assert "\t" not in line, f"Line {num + 1} has tabs"
 
 
+@pytest.mark.parametrize(
+    "args",
+    [
+        ["--input", TEST_MODEL],
+        ["--input", TEST_MODEL, "--context-url", TEST_CONTEXT, SPDX3_CONTEXT_URL],
+    ],
+)
+class TestCheckType:
+    """
+    Static type checking tests for the generated Python code
+    """
+
+    def test_mypy(self, tmp_path, args):
+        """
+        Mypy static type checking
+        """
+        outfile = tmp_path / "output.py"
+        shacl2code_generate(args, outfile)
+        subprocess.run(
+            ["mypy", outfile],
+            encoding="utf-8",
+            check=True,
+        )
+
+    def test_pyrefly(self, tmp_path, args):
+        """
+        Pyrefly static type checking
+        """
+        outfile = tmp_path / "output.py"
+        shacl2code_generate(args, outfile)
+        subprocess.run(
+            ["pyrefly", "check", outfile],
+            encoding="utf-8",
+            check=True,
+        )
+
+    def test_pyright(self, tmp_path, args):
+        """
+        Pyright static type checking
+        """
+        outfile = tmp_path / "output.py"
+        shacl2code_generate(args, outfile)
+        subprocess.run(
+            ["pyright", outfile],
+            encoding="utf-8",
+            check=True,
+        )
+
+
+def check_file(p, expect, digest):
+    sha1 = hashlib.sha1()
+    with p.open("rb") as f:
+        while True:
+            d = f.read(4096)
+            if not d:
+                break
+            sha1.update(d)
+
+    assert sha1.hexdigest() == digest
+
+    with p.open("r") as f:
+        data = json.load(f)
+
+    assert data == expect
+
+
 def test_roundtrip(model, tmp_path, roundtrip):
-    def check_file(p, expect, digest):
-        sha1 = hashlib.sha1()
-        with p.open("rb") as f:
-            while True:
-                d = f.read(4096)
-                if not d:
-                    break
-                sha1.update(d)
-
-        assert sha1.hexdigest() == digest
-
-        with p.open("r") as f:
-            data = json.load(f)
-
-        assert data == expect
-
     doc = model.SHACLObjectSet()
     with roundtrip.open("r") as f:
         d = model.JSONLDDeserializer()
@@ -209,6 +236,51 @@ def test_script_roundtrip(test_script, tmp_path, roundtrip):
     assert data == expect_data
 
 
+def test_from_rdf_roundtrip(model, tmp_path, roundtrip):
+    with roundtrip.open("r") as f:
+        expect_data = json.load(f)
+
+    # Parse data using RDF
+    g = rdflib.Graph()
+    g.parse(roundtrip)
+
+    # Convert to SHACL objects
+    objset = model.SHACLObjectSet()
+    model.RDFDeserializer().read(g, objset)
+
+    # Write out
+    outfile = tmp_path / "out.json"
+    with outfile.open("wb") as f:
+        digest = model.JSONLDInlineSerializer().write(objset, f)
+
+    check_file(outfile, expect_data, digest)
+
+
+def test_to_rdf_roundtrip(model, tmp_path, roundtrip):
+    with roundtrip.open("r") as f:
+        expect_data = json.load(f)
+
+    # Read JSON data
+    objset = model.SHACLObjectSet()
+    with roundtrip.open("r") as f:
+        model.JSONLDDeserializer().read(f, objset)
+
+    # Convert to RDF
+    g = rdflib.Graph()
+    model.RDFSerializer().write(objset, g)
+
+    # Convert from RDF to new object set
+    objset = model.SHACLObjectSet()
+    model.RDFDeserializer().read(g, objset)
+
+    # Write out
+    outfile = tmp_path / "out.json"
+    with outfile.open("wb") as f:
+        digest = model.JSONLDInlineSerializer().write(objset, f)
+
+    check_file(outfile, expect_data, digest)
+
+
 def test_jsonschema_validation(roundtrip, test_jsonschema):
     with roundtrip.open("r") as f:
         data = json.load(f)
@@ -1068,58 +1140,70 @@ def test_extensible_prop(model, test_context_url, prop, serkey, value, expect):
 
 
 @pytest.mark.parametrize(
-    "iri,value,expect,ser_expect",
+    "iri,value,expect,ser_data,ser_expect",
     [
         (
             "extensible_class_property",
             "foo",
             KeyError,
             None,
+            None,
         ),
         (
             "http://example.org/extensible-test-prop",
             "foo",
             SAME_AS_VALUE,
-            SAME_AS_VALUE,
+            ["foo"],
+            ["foo"],
         ),
         (
             "http://example.org/extensible-test-prop",
             1,
             SAME_AS_VALUE,
-            SAME_AS_VALUE,
+            [1],
+            [1],
         ),
         (
             "http://example.org/extensible-test-prop",
             1.123,
             SAME_AS_VALUE,
-            "1.123",
+            ["1.123"],
+            [1.123],
         ),
         (
             "http://example.org/extensible-test-prop",
             object(),
             SAME_AS_VALUE,
             TypeError,
+            None,
         ),
         (
             "http://example.org/extensible-test-prop",
             [1, "foo", 1.123],
             SAME_AS_VALUE,
             [1, "foo", "1.123"],
+            [1, "foo", 1.123],
         ),
         (
             "http://example.org/extensible-test-prop",
             [object()],
             SAME_AS_VALUE,
             TypeError,
+            None,
         ),
     ],
 )
-def test_extensible_iri(model, test_context_url, iri, value, expect, ser_expect):
+def test_extensible_iri(
+    model, test_context_url, iri, value, expect, ser_data, ser_expect
+):
     e = model.extensible_class(extensible_class_required="required")
 
     if expect is SAME_AS_VALUE:
         expect = value
 
+    if ser_data is SAME_AS_VALUE:
+        ser_data = value
+
     if ser_expect is SAME_AS_VALUE:
         ser_expect = value
 
@@ -1135,8 +1219,8 @@ def test_extensible_iri(model, test_context_url, iri, value, expect, ser_expect)
         objset = model.SHACLObjectSet()
         objset.add(e)
 
-        if isinstance(ser_expect, type) and issubclass(ser_expect, Exception):
-            with pytest.raises(ser_expect):
+        if isinstance(ser_data, type) and issubclass(ser_data, Exception):
+            with pytest.raises(ser_data):
                 data = s.serialize_data(objset)
         else:
             data = s.serialize_data(objset)
@@ -1144,7 +1228,7 @@ def test_extensible_iri(model, test_context_url, iri, value, expect, ser_expect)
                 "@context": test_context_url,
                 "@type": "extensible-class",
                 "extensible-class/required": "required",
-                iri: ser_expect,
+                iri: ser_data,
             }
 
             objset = model.SHACLObjectSet()
@@ -1152,7 +1236,7 @@ def test_extensible_iri(model, test_context_url, iri, value, expect, ser_expect)
             d.deserialize_data(data, objset)
 
             e = objset.objects.pop()
-            assert e[iri] == expect
+            assert e[iri] == ser_expect
 
         del e[iri]
         with pytest.raises(KeyError):
